/*
 * SPI testing utility (using spidev driver)
 *
 * Copyright (c) 2007  MontaVista Software, Inc.
 * Copyright (c) 2007  Anton Vorontsov <avorontsov@ru.mvista.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * Cross-compile with cross-gcc -I/path/to/cross-kernel/include
 */

#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>
#include <string.h>

	enum{ CH0,CH1,CH2,CH3,CH4,CH5,CH6,CH7};

	#define MODE_SINGLE	(1<<3)
	#define MODE_DIFF		0


#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))

static void pabort(const char *s)
{
	perror(s);
	abort();
}

static const char *device = "/dev/spidev1.0";
static uint8_t mode;
static uint8_t bits = 8;//transfer(fd,ret)*3.3/4096;
static uint32_t speed = 500000;
static uint16_t delay;

static int transfer(int fd, uint8_t ch)
{
	int ret;
	uint16_t aux;
	uint8_t tx[3];


	/*
		MSB  	| 0  0  0  0  - 0  S  M  D2  | 	wr		//write Start,Mode,D2
				| D1 D0 0  0  - b11..... b8  | 	wr/rd	//write D1,D0. Read null, B11-B8
		LSB  	| b7......b4  - b3.......b0  |  rd		//read B7-B0
	*/
	#define D0 				(1<<0)
	#define D1 				(1<<1)
	#define D2		 		(1<<2)
	#define SGL_MOD	 		(1<<3)
	#define START		 	(1<<4)

		aux = ( START | MODE_SINGLE | ch ) << 6;	//build 0000  0SMD DD00  0000
		tx[0] = *(((char*)&aux)+1);
		tx[1] = *(((char*)&aux)+0);
		tx[2] = 0xFF;
		//tx[3] = 0xFF;


	uint8_t rx[ARRAY_SIZE(tx)] = {0, };
	struct spi_ioc_transfer tr = {
		.tx_buf = (unsigned long)tx,
		.rx_buf = (unsigned long)rx,
		.len = ARRAY_SIZE(tx),
		.delay_usecs = delay,
		.speed_hz = speed,
		.bits_per_word = bits,
	};

	ret = ioctl(fd, SPI_IOC_MESSAGE(1), &tr);
	if (ret < 0)
		pabort("can't send spi message");

	ret = ((rx[1]&0x0f)<<8) + rx[2];//Clean the upper nibble of MSB
	return ret;

}

void print_usage(const char *prog)
{
	printf("Usage: %s [-DsbdlHOLC3]\n", prog);
	puts("  -D --device   device to use (default /dev/spidev1.1)\n"
	     "  -s --speed    max speed (Hz)\n"
	     "  -d --delay    delay (usec)\n"
	     "  -b --bpw      bits per word \n"
	     "  -l --loop     loopback\n"
	     "  -H --cpha     clock phase\n"
	     "  -O --cpol     clock polarity\n"
	     "  -L --lsb      least significant bit first\n"
	     "  -C --cs-high  chip select active high\n"
	     "  -3 --3wire    SI/SO signals shared\n");
	exit(1);
}

void parse_opts(int argc, char *argv[])
{
	while (1) {
		static const struct option lopts[] = {
			{ "device",  1, 0, 'D' },
			{ "speed",   1, 0, 's' },
			{ "delay",   1, 0, 'd' },
			{ "bpw",     1, 0, 'b' },
			{ "loop",    0, 0, 'l' },
			{ "cpha",    0, 0, 'H' },
			{ "cpol",    0, 0, 'O' },
			{ "lsb",     0, 0, 'L' },
			{ "cs-high", 0, 0, 'C' },
			{ "3wire",   0, 0, '3' },
			{ NULL, 0, 0, 0 },
		};
		int c;

		c = getopt_long(argc, argv, "D:s:d:b:lHOLC3", lopts, NULL);

		if (c == -1)
			break;

		switch (c) {
		case 'D':
			device = optarg;
			break;
		case 's':
			speed = atoi(optarg);
			break;
		case 'd':
			delay = atoi(optarg);
			break;
		case 'b':
			bits = atoi(optarg);
			break;
		case 'l':
			mode |= SPI_LOOP;
			break;
		case 'H':
			mode |= SPI_CPHA;
			break;
		case 'O':
			mode |= SPI_CPOL;
			break;
		case 'L':
			mode |= SPI_LSB_FIRST;
			break;
		case 'C':
			mode |= SPI_CS_HIGH;
			break;
		case '3':
			mode |= SPI_3WIRE;
			break;
		default:
			print_usage(argv[0]);
			break;
		}
	}
}

int main(int argc, char *argv[])
{
	int ret = 0;
	int fd;
	float val;
	FILE * flog;
	int aux;
	unsigned long v[8];

	parse_opts(argc, argv);

	fd = open(device, O_RDWR);
	if (fd < 0)
		pabort("can't open device");

	/*
	 * spi mode
	 */
	ret = ioctl(fd, SPI_IOC_WR_MODE, &mode);
	if (ret == -1)
		pabort("can't set spi mode");

	ret = ioctl(fd, SPI_IOC_RD_MODE, &mode);
	if (ret == -1)
		pabort("can't get spi mode");

	/*
	 * bits per word
	 */
	ret = ioctl(fd, SPI_IOC_WR_BITS_PER_WORD, &bits);
	if (ret == -1)
		pabort("can't set bits per word");

	ret = ioctl(fd, SPI_IOC_RD_BITS_PER_WORD, &bits);
	if (ret == -1)
		pabort("can't get bits per word");

	/*
	 * max speed hz
	 */
	ret = ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
	if (ret == -1)
		pabort("can't set max speed hz");

	ret = ioctl(fd, SPI_IOC_RD_MAX_SPEED_HZ, &speed);
	if (ret == -1)
		pabort("can't get max speed hz");

	printf("spi mode: %d\n", mode);
	printf("bits per word: %d\n", bits);
	printf("max speed: %d Hz (%d KHz)\nOk, now to work...\n", speed, speed/1000);


	while(1){
		aux=100;
		memset(v,0,sizeof(v));
		while(aux--){
			for (ret=0;ret<8;ret++)
				v[ret]+=transfer(fd,ret);
		}

		flog = fopen("/tmp/values.xml", "w");
		fputs("<?xml version='1.0' encoding='ISO-8859-1'?>\r\n<values>\r\n\t<analog>\r\n", flog);
		for (ret=0;ret<8;ret++){
			val = v[ret]*3.3/409600;
			fprintf(flog,"\t\t<ch>%.04f</ch>\r\n", val);
		}
		fputs("\t</analog>\r\n</values>\r\n",flog);
		fclose(flog);
	}
	fclose(flog);
	close(fd);

	return ret;


}
